"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Freezer general utils functions
"""
import threading
import logging
import Queue


class Wait(Exception):
    pass


class StorageManager:

    def __init__(self, input_queue, output_queues):
        """
        :type input_queue: streaming.RichQueue
        :param input_queue:
        :type output_queues: collections.Iterable[streaming.RichQueue]
        :param output_queues:
        :return:
        """
        self.input_queue = input_queue
        self.output_queues = output_queues
        self.broken_output_queues = set()

    def send_message(self, message, finish=False):
        for output_queue in self.output_queues:
            if output_queue not in self.broken_output_queues:
                try:
                    if finish:
                        output_queue.finish()
                    else:
                        output_queue.put(message)
                except Exception as e:
                    logging.exception(e)
                    StorageManager.one_fails_all_fail(
                        self.input_queue, self.output_queues)
                    self.broken_output_queues.add(output_queue)

    def transmit(self):
        for message in self.input_queue.get_messages():
            self.send_message(message)
        self.send_message("", True)

    @staticmethod
    def one_fails_all_fail(input_queue, output_queues):
        input_queue.force_stop()
        for output_queue in output_queues:
            output_queue.force_stop()
        raise Exception("All fail")


class RichQueue:
    """
        :type data_queue: Queue.Queue
    """
    def __init__(self, size=2):
        """
        :type size: int
        :return:
        """
        self.data_queue = Queue.Queue(maxsize=size)
        # transmission changes in atomic way so no synchronization needed
        self.finish_transmission = False
        self.is_force_stop = False

    def finish(self):
        self.finish_transmission = True

    def force_stop(self):
        self.is_force_stop = True

    def empty(self):
        return self.data_queue.empty()

    def get(self):
        try:
            res = self.data_queue.get(timeout=1)
            self.data_queue.task_done()
            return res
        except Queue.Empty:
            raise Wait()

    def check_stop(self):
        if self.is_force_stop:
            raise Exception("Forced stop")

    def put_messages(self, messages):
        for message in messages:
            self.put(message)
        self.finish()

    def has_more(self):
        self.check_stop()
        return not self.finish_transmission or not self.data_queue.empty()

    def put(self, message):
        while True:
            try:
                self.data_queue.put(message, timeout=1)
                break
            except Queue.Full:
                self.check_stop()

    def get_messages(self):
        while self.has_more():
            try:
                yield self.get()
            except Wait:
                self.check_stop()


class QueuedThread(threading.Thread):
    def __init__(self, target, rich_queue, args=(), kwargs=None):
        """
            :type args: collections.Iterable
            :type kwargs: dict
            :type target: () -> ()
            :type rich_queue: RichQueue
            """
        self.args = args
        kwargs = kwargs or {}
        self.rich_queue = rich_queue
        kwargs["rich_queue"] = rich_queue
        super(QueuedThread, self).__init__(target=target, args=args,
                                           kwargs=kwargs)

    def run(self):
        try:
            super(QueuedThread, self).run()
        except Exception as e:
            self.rich_queue.force_stop()
            raise e


def stream(read_function, read_function_kwargs,
           write_function, write_function_kwargs, queue_size=10):
    """
    :param queue_size:
    :type queue_size: int
    :return:
    """
    input_queue = RichQueue(queue_size)
    read_stream = QueuedThread(read_function, input_queue,
                               kwargs=read_function_kwargs)
    output_queue = RichQueue(queue_size)
    write_stream = QueuedThread(write_function, output_queue,
                                kwargs=write_function_kwargs)
    read_stream.daemon = True
    write_stream.daemon = True
    read_stream.start()
    write_stream.start()
    manager = StorageManager(input_queue, [output_queue])
    manager.transmit()
    read_stream.join()
    write_stream.join()



"""Freezer main.py related tests

(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from commons import *
from freezer import (restore, backup, exec_cmd)
from freezer.job import (
    Job, InfoJob, AdminJob, BackupJob, RestoreJob, ExecJob, create_job)
from freezer import (restore, backup)

from freezer.job import Job, InfoJob, AdminJob, BackupJob, RestoreJob, create_job
import logging
from mock import patch, Mock
import unittest


class TestJob:

    def do_monkeypatch(self, monkeypatch):
        self.fakebackup = FakeBackup()

    def test_execute(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        job = Job(BackupOpt1())
        assert job.execute() is None


class TestInfoJob(TestJob):

    def test_execute_nothing_to_do(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        job = InfoJob(backup_opt)
        job.execute()

    def test_execute_list_containers(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        backup_opt.list_containers = True
        job = InfoJob(backup_opt)
        job.execute()


class TestBackupJob(TestJob):

    def test_execute_backup_fs_no_incremental_and_backup_level_raise(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        backup_opt.mode = 'fs'
        backup_opt.no_incremental = True
        job = BackupJob(backup_opt)
        pytest.raises(Exception, job.execute)

    def test_execute_backup_mongo(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        monkeypatch.setattr(backup, 'backup_mode_mongo', self.fakebackup.fake_backup_mode_mongo)
        backup_opt = BackupOpt1()
        backup_opt.no_incremental = False
        backup_opt.mode = 'mongo'
        job = BackupJob(backup_opt)
        assert job.execute() is None

    def test_execute_backup_mysql(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        monkeypatch.setattr(backup, 'backup_mode_mysql', self.fakebackup.fake_backup_mode_mysql)
        backup_opt = BackupOpt1()
        backup_opt.no_incremental = False
        backup_opt.mode = 'mysql'
        job = BackupJob(backup_opt)
        assert job.execute() is None

    def test_execute_raise(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        backup_opt.no_incremental = False
        backup_opt.mode = None
        job = BackupJob(backup_opt)
        pytest.raises(ValueError, job.execute)


class TestAdminJob(TestJob):
    def test_execute(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        job = AdminJob(backup_opt)
        assert job.execute() is None


class TestExecJob(TestJob):

    def setUp(self):
        #init mock_popen
        self.popen=patch('freezer.exec_cmd.subprocess.Popen')
        self.mock_popen=self.popen.start()
        self.mock_popen.return_value = Mock()
        self.mock_popen.return_value.communicate = Mock()
        self.mock_popen.return_value.communicate.return_value = ['some stderr']

    def tearDown(self):
        self.popen.stop()

    def test_execute_nothing_to_do(self, monkeypatch):
        self.do_monkeypatch(monkeypatch)
        backup_opt = BackupOpt1()
        job = ExecJob(backup_opt)
        assert job.execute() is False

    def test_execute_script(self, monkeypatch):
        self.setUp()
        self.do_monkeypatch(monkeypatch)
        self.mock_popen.return_value.returncode = 0
        backup_opt = BackupOpt1()
        backup_opt.command='echo test'
        job = ExecJob(backup_opt)
        assert job.execute() is True
        self.tearDown()

    def test_execute_raise(self, monkeypatch):
        self.setUp()
        self.do_monkeypatch(monkeypatch)
        popen=patch('freezer.exec_cmd.subprocess.Popen')
        self.mock_popen.return_value.returncode = 1
        backup_opt = BackupOpt1()
        backup_opt.command='echo test'
        job = ExecJob(backup_opt)
        pytest.raises(Exception, job.execute)
        self.tearDown()


def test_create_job():
    backup_opt = BackupOpt1()
    backup_opt.action = None
    pytest.raises(Exception, create_job, backup_opt)

    backup_opt.action = 'backup'
    job = create_job(backup_opt)
    assert isinstance(job, BackupJob)

    backup_opt.action = 'restore'
    job = create_job(backup_opt)
    assert isinstance(job, RestoreJob)

    backup_opt.action = 'info'
    job = create_job(backup_opt)
    assert isinstance(job, InfoJob)

    backup_opt.action = 'admin'
    job = create_job(backup_opt)
    assert isinstance(job, AdminJob)

    backup_opt.action = 'exec'
    job = create_job(backup_opt)
    assert isinstance(job, ExecJob)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest
from mock import Mock, patch

from freezer.scheduler import arguments


class TestBaseParser(unittest.TestCase):

    def test_returns_parser(self):
        mock_parser = Mock()
        mock_parser._me = 'test12345'
        retval = arguments.base_parser(mock_parser)
        self.assertEquals(retval._me, 'test12345')

        call_count = mock_parser.add_argument.call_count
        self.assertGreater(call_count, 10)

    @patch('freezer.scheduler.arguments.base_parser')
    def test_get_args_return_parsed_args(self, mock_base_parser):
        mock_parser = Mock()
        mock_parser.parse_args.return_value = 'pluto'
        mock_base_parser.return_value = mock_parser
        retval = arguments.get_args(['alpha', 'bravo'])
        call_count = mock_parser.add_argument.call_count
        self.assertGreater(call_count, 0)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from freezer import  exec_cmd
from mock import patch, Mock
import subprocess

from __builtin__ import True


def test_exec_cmd(monkeypatch):
    cmd="echo test > test.txt"
    popen=patch('freezer.exec_cmd.subprocess.Popen')
    mock_popen=popen.start()
    mock_popen.return_value = Mock()
    mock_popen.return_value.communicate = Mock()
    mock_popen.return_value.communicate.return_value = ['some stderr']
    mock_popen.return_value.returncode = 0
    exec_cmd.execute(cmd)
    assert (mock_popen.call_count == 1)
    mock_popen.assert_called_with(['echo', 'test', '>', 'test.txt'],
                                   shell=False,
                                   stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE)
    popen.stop()


def test__exec_cmd_with_pipe(monkeypatch):
    cmd="echo test|wc -l"
    popen=patch('freezer.exec_cmd.subprocess.Popen')
    mock_popen=popen.start()
    mock_popen.return_value = Mock()
    mock_popen.return_value.communicate = Mock()
    mock_popen.return_value.communicate.return_value = ['some stderr']
    mock_popen.return_value.returncode = 0
    exec_cmd.execute(cmd)
    assert (mock_popen.call_count == 2)
    popen.stop()



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from freezer.winutils import is_windows
from freezer.winutils import use_shadow
from freezer.winutils import clean_tar_command
from freezer.winutils import add_gzip_to_command
from freezer.winutils import DisableFileSystemRedirection
from freezer import winutils
from commons import *
import logging


class TestWinutils:

    def test_is_windows(self, monkeypatch):
        fake_os = Os()
        monkeypatch.setattr(os, 'name', fake_os)
        assert is_windows() is False

    def test_use_shadow(self):
        test_volume = 'C:'
        test_volume2 = 'C:\\'
        path = 'C:\\Users\\Test'
        expected = 'C:\\freezer_shadowcopy\\Users\\Test'
        assert use_shadow(path, test_volume2) == expected

        # test if the volume format is incorrect
        pytest.raises(Exception, use_shadow(path, test_volume))

    def test_clean_tar_command(self):
        test_tar_command = 'tar --create -z --warning=none ' \
                           '--no-check-device --one-file-system ' \
                           '--preserve-permissions --same-owner --seek ' \
                           '--ignore-failed-read '
        expected = 'tar --create    --one-file-system --preserve-permissions ' \
                   '--same-owner  --ignore-failed-read '

        assert clean_tar_command(test_tar_command) == expected

    def test_add_gzip_to_command(self):
        test_command = 'tar --create    --one-file-system ' \
                       '--preserve-permissions --same-owner ' \
                       '--ignore-failed-read '
        expected = 'tar --create    --one-file-system ' \
                   '--preserve-permissions --same-owner ' \
                   '--ignore-failed-read  | gzip -7'

        assert add_gzip_to_command(test_command) == expected

    def test_DisableFileSystemRedirection(self, monkeypatch):
        fake_disable_redirection = DisableFileSystemRedirection()
        fake_disable_redirection.success = True
        assert fake_disable_redirection._revert == ''
        assert fake_disable_redirection._disable == ''

        pytest.raises(Exception, fake_disable_redirection.__enter__)
        pytest.raises(Exception, fake_disable_redirection.__exit__)

    def test_start_sql_server(self, monkeypatch):
        fake_disable_redirection = FakeDisableFileSystemRedirection()
        backup_opt = BackupOpt1()
        fakesubprocess = FakeSubProcess()
        fakesubprocesspopen = fakesubprocess.Popen()

        monkeypatch.setattr(
            subprocess.Popen, 'communicate',
            fakesubprocesspopen.communicate)
        monkeypatch.setattr(
            subprocess, 'Popen', fakesubprocesspopen)
        monkeypatch.setattr(
            winutils.DisableFileSystemRedirection, '__enter__',
            fake_disable_redirection.__enter__)
        monkeypatch.setattr(
            winutils.DisableFileSystemRedirection, '__exit__',
            fake_disable_redirection.__exit__)

        assert winutils.start_sql_server(backup_opt) is not False

        fakesubprocess = FakeSubProcess3()
        fakesubprocesspopen = fakesubprocess.Popen()

        monkeypatch.setattr(
            subprocess.Popen, 'communicate',
            fakesubprocesspopen.communicate)
        monkeypatch.setattr(
            subprocess, 'Popen', fakesubprocesspopen)

        pytest.raises(
            Exception,
            winutils.start_sql_server(backup_opt.sql_server_instance))

    def test_stop_sql_server(self, monkeypatch):
        fake_disable_redirection = FakeDisableFileSystemRedirection()
        backup_opt = BackupOpt1()
        fakesubprocess = FakeSubProcess()
        fakesubprocesspopen = fakesubprocess.Popen()

        monkeypatch.setattr(
            subprocess.Popen, 'communicate',
            fakesubprocesspopen.communicate)
        monkeypatch.setattr(
            subprocess, 'Popen', fakesubprocesspopen)
        monkeypatch.setattr(
            winutils.DisableFileSystemRedirection, '__enter__',
            fake_disable_redirection.__enter__)
        monkeypatch.setattr(
            winutils.DisableFileSystemRedirection, '__exit__',
            fake_disable_redirection.__exit__)

        assert winutils.start_sql_server(
            backup_opt.sql_server_instance) is not False

        fakesubprocess = FakeSubProcess3()
        fakesubprocesspopen = fakesubprocess.Popen()

        monkeypatch.setattr(
            subprocess.Popen, 'communicate',
            fakesubprocesspopen.communicate)
        monkeypatch.setattr(
            subprocess, 'Popen', fakesubprocesspopen)

        pytest.raises(Exception, winutils.stop_sql_server(
            backup_opt.sql_server_instance))



#(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.

from setuptools import setup
import sys
import subprocess

# All this Machinery unfortunately is needed to support win32 platform.
# We follow the advice from pbr as:
# Note that we do nott support the easy_install aspects of setuptools:
# while we depend on setup_requires, for any install_requires we recommend
# that they be installed prior to running setup.py install - either by hand,
# or by using an install tool such as pip.
install_requires = ''
if sys.platform.startswith('linux'):
    install_requires = 'pep3143daemon'
elif sys.platform.startswith('win32'):
    install_requires = 'pywin32'

subprocess.call(['easy_install', 'pip'])
if install_requires:
  subprocess.call(['pip', 'install', install_requires])

setup(
    setup_requires='pbr>=0.6,!=0.7,<1.0',
    pbr=True
)




# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from freezer.storage import fslike
from freezer.storage import storage
import mock

class TestFsLikeStorage(object):
    def test_download_meta_file(self, tmpdir):
        t = fslike.FsLikeStorage(tmpdir.strpath, tmpdir.strpath)
        full_backup = storage.Backup("test", 2000)
        increment = storage.Backup("test", 2500, 9, full_backup)
        full_backup.add_increment(increment)
        backup = storage.Backup("test", 3000, 10, full_backup)
        t.get_file = mock.Mock()
        t.download_meta_file(backup)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logging
import time

from cinderclient import client as cclient
from glanceclient import client as gclient
from novaclient import client as nclient
import swiftclient

from utils import Bunch
from utils import ReSizeStream


class ClientManager:
    """
    :type swift: swiftclient.Connection
    :type glance: glanceclient.v1.client.Client
    :type nova: novaclient.v2.client.Client
    :type cinder: cinderclient.v1.client.Client
    """
    def __init__(self, options, insecure=True,
                 swift_auth_version=2, dry_run=False):
        """
        Creates manager of connections to swift, nova, glance and cinder
        :param options: OpenstackOptions
        :type options: freezer.utils.OpenstackOptions
        :param insecure:
        :param swift_auth_version:
        :param dry_run:
        :return:
        """
        self.options = options
        self.insecure = insecure
        self.swift_auth_version = swift_auth_version
        self.dry_run = dry_run
        self.cinder = None
        self.swift = None
        self.glance = None
        self.nova = None

    def get_cinder(self):
        """
        :rtype cinderclient.v1.client.Client
        :return:
        """
        if not self.cinder:
            self.create_cinder()
        return self.cinder

    def get_swift(self):
        """
        :rtype swiftclient.Connection
        :return: instance of swift client
        """
        if not self.swift:
            self.create_swift()
        return self.swift

    def get_glance(self):
        """
        :rtype glanceclient.v1.client.Client
        :return:
        """
        if not self.glance:
            self.create_glance()
        return self.glance

    def get_nova(self):
        """
        :rtype
        :return:
        """
        if not self.nova:
            self.create_nova()
        return self.nova

    def create_cinder(self):
        """
        Creates client for cinder and caches it
        :rtype cinderclient.v1.client.Client
        :return: instance of cinder client
        """
        options = self.options
        logging.info("[*] Creation of cinder client")
        self.cinder = cclient.Client(
            version="1",
            username=options.user_name,
            api_key=options.password,
            project_id=options.tenant_name,
            auth_url=options.auth_url,
            region_name=options.region_name,
            insecure=self.insecure,
            endpoint_type=options.endpoint_type or 'publicURL',
            service_type="volume")
        return self.cinder

    def create_swift(self):
        """
        Creates client for swift and caches it
        :rtype swiftclient.Connection
        :return: instance of swift client
        """
        options = self.options
        logging.info("[*] Creation of swift client")

        self.swift = swiftclient.client.Connection(
            authurl=options.auth_url,
            user=options.user_name, key=options.password,
            tenant_name=options.tenant_name,
            os_options=options.os_options,
            auth_version=self.swift_auth_version,
            insecure=self.insecure, retries=6)

        if self.dry_run:
            self.swift = DryRunSwiftclientConnectionWrapper(self.swift)
        return self.swift

    def create_glance(self):
        """
        Creates client for glance and caches it
        :rtype glanceclient.v1.client.Client
        :return: instance of glance client
        """

        from glanceclient.shell import OpenStackImagesShell

        options = self.options

        logging.info("[*] Creation of glance client")

        endpoint, token = OpenStackImagesShell()._get_endpoint_and_token(
            Bunch(os_username=options.user_name,
                  os_password=options.password,
                  os_tenant_name=options.tenant_name,
                  os_project_name=options.project_name,
                  os_auth_url=options.auth_url,
                  os_region_name=options.region_name,
                  endpoint_type=options.endpoint_type,
                  force_auth=False))

        self.glance = gclient.Client(version="1",
                                     endpoint=endpoint, token=token)
        return self.glance

    def create_nova(self):
        """
        Creates client for nova and caches it
        :return:
        """
        options = self.options
        logging.info("[*] Creation of nova client")

        self.nova = nclient.Client(
            version='2',
            username=options.user_name,
            api_key=options.password,
            project_id=options.tenant_name,
            auth_url=options.auth_url,
            region_name=options.region_name,
            insecure=self.insecure)

        return self.nova

    def provide_snapshot(self, volume, snapshot_name):
        """
        Creates snapshot for cinder volume with --force parameter
        :param volume: volume object for snapshoting
        :param snapshot_name: name of snapshot
        :return: snapshot object
        """
        snapshot = self.get_cinder().volume_snapshots.create(
            volume_id=volume.id,
            display_name=snapshot_name,
            force=True)

        logging.debug("Snapshot for volume with id {0}".format(volume.id))

        while snapshot.status != "available":
            try:
                logging.debug("Snapshot status: " + snapshot.status)
                snapshot = self.get_cinder().volume_snapshots.get(snapshot.id)
                if snapshot.status == "error":
                    raise Exception("snapshot has error state")
                time.sleep(5)
            except Exception as e:
                if e.message == "snapshot has error state":
                    raise e
                logging.exception(e)
        return snapshot

    def do_copy_volume(self, snapshot):
        """
        Creates new volume from a snapshot
        :param snapshot: provided snapshot
        :return: created volume
        """
        volume = self.get_cinder().volumes.create(
            size=snapshot.size,
            snapshot_id=snapshot.id)

        while volume.status != "available":
            try:
                logging.info("[*] Volume copy status: " + volume.status)
                volume = self.get_cinder().volumes.get(volume.id)
                time.sleep(5)
            except Exception as e:
                logging.exception(e)
                logging.warn("[*] Exception getting volume status")
        return volume

    def make_glance_image(self, image_volume_name, copy_volume):
        """
        Creates an glance image from volume
        :param image_volume_name: Name of image
        :param copy_volume: volume to make an image
        :return: Glance image object
        """
        image_id = self.get_cinder().volumes.upload_to_image(
            volume=copy_volume,
            force=True,
            image_name=image_volume_name,
            container_format="bare",
            disk_format="raw")[1]["os-volume_upload_image"]["image_id"]
        image = self.get_glance().images.get(image_id)
        while image.status != "active":
            try:
                time.sleep(5)
                logging.info("Image status: " + image.status)
                image = self.get_glance().images.get(image.id)
                if image.status in ("killed", "deleted"):
                    raise Exception("Image have killed state")
            except Exception as e:
                if image.status in ("killed", "deleted"):
                    raise e
                logging.exception(e)
                logging.warn("Exception getting image status")
        return image

    def clean_snapshot(self, snapshot):
        """
        Deletes snapshot
        :param snapshot: snapshot name
        """
        logging.info("[*] Deleting existed snapshot: " + snapshot.id)
        self.get_cinder().volume_snapshots.delete(snapshot)

    def download_image(self, image):
        """
        Creates a stream for image data
        :param image: Image object for downloading
        :return: stream of image data
        """
        logging.debug("Download image enter")
        stream = self.get_glance().images.data(image.id)
        logging.debug("Stream with size {0}".format(image.size))
        return ReSizeStream(stream, image.size, 1000000)


class DryRunSwiftclientConnectionWrapper:
    def __init__(self, sw_connector):
        self.sw_connector = sw_connector
        self.get_object = sw_connector.get_object
        self.get_account = sw_connector.get_account
        self.get_container = sw_connector.get_container
        self.head_object = sw_connector.head_object
        self.put_object = self.dummy
        self.put_container = self.dummy
        self.delete_object = self.dummy

    def dummy(self, *args, **kwargs):
        pass



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

from freezer.storage import ssh

class TestSshStorage(unittest.TestCase):

    def test_constructor(self):
        ssh.SshStorage.init = lambda x: x
        ssh.SshStorage(
            "test_dir", "test_work_dir", "test_key", "test_name", "test_ip",
            "teset_port")

    def test_info(self):
        ssh.SshStorage.init = lambda x: x
        ssh.SshStorage(
            "test_dir", "test_work_dir", "test_key", "test_name", "test_ip",
            "teset_port").info()
